@@ -52,7 +52,7 @@ namespace solve_gmp_R
    * A is of dimension nxn X nxm and B nxm (X will be return a B address)
    * We use the Gauss algorithm
    */
-  template< class T> void solve (matrix::Matrix<T> & A , matrix::Matrix<T> & B)
+  template< class T> void solve (math::Matrix<T> & A , math::Matrix<T> & B)
     {
 
       // A [ i ,j] = A[ i + j * A.nrow]
