@@ -111,7 +111,7 @@ for(fid in allfunctionid)
       if(nfid != fid) cat("-> num.fn.:", nfid)
       cat("\n-> all.equal(target = res, current = F(<numeric x>)):\n")
       nres <- test(FUN, nx, out = "numeric")
-      print(all.equal(res, nres))
+     # print(all.equal(res, nres))
     } else cat("\n\n")
     print(as.data.frame(rc)); cat("\n")
     ##    ^^^^^^^^^^^^^ (for now, to diminuish difference to last version )
